<?php

//set_magic_quotes_runtime(0);

//
// globally set the permissions level, so someone can be more or less restrictive if they want
// 
global $perm_level;
$perm_level = 0777;

//
// start session up
//
session_cache_limiter('none');
session_start();
session_name("PHPSESSID");

require_once("legacylib.php");
require_once("datastore.php");
require_once("seeder.php");


/*
	there's a couple functions in here that we might want to use
	without actually setting up BM, so we can use this flag to skip
	setup
*/
global $skip_setup;
if ( !( isset($skip_setup) && $skip_setup == 1 ) ) {
  setup_data_directories();
}



//
// if the user has logged in through http auth, but doesn't have cookies on, or
// something like that, then this will grab their login info and put it in the
// session array
//
global $can_use_cookies;
$can_use_cookies = true;

if ( !isset($_SESSION["user"]) && 
		isset($_SERVER['PHP_AUTH_USER']) && $_SERVER['PHP_AUTH_USER'] &&
	login($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'], $errstr) ) {

	$can_use_cookies = false;

}    

// if we are in this particular setup, then that means the user
// can't use cookies - so we won't bother showing them a logout link,
// since they can't logout
//if ( !isset($_SESSION['cookietest']) ) {
//	$can_use_cookies = false;
//}


function version_number() {
	return 10;
}

function get_version() {
  return 'Preview Release ' . version_number();
}

//Returns the base url of this tracker
function get_base_url() {

// cjm try using this instead
//http://".$_SERVER['HTTP_HOST']
//                      .dirname($_SERVER['PHP_SELF'])
//                      ."/".$relative_url)

	if ((!isset($_SERVER['HTTPS'])) && $_SERVER['SERVER_PORT']!=80)
		$port = ':'.$_SERVER['SERVER_PORT'];
	elseif ((isset($_SERVER['HTTPS']) && $_SERVER['SERVER_PORT']!=443))
		$port = ':'.$_SERVER['SERVER_PORT'];
	else
		$port = '';
	
	$url = 'http'.( isset($_SERVER['HTTPS']) ? 's' : '') . '://' . $_SERVER['HTTP_HOST'] . $port .
		preg_replace( '|^(.*/).*$|','\\1', $_SERVER['PHP_SELF'] );
	
	// make sure the url doesnt have multiple slashes on the end (bug # 1211743)
	while ( $url{strlen($url)-1} == '/' ) {
		$url = substr($url, 0, strlen($url) - 1);
	}
	
	// make sure the url ends with a slash
	if ( $url{strlen($url)-1} != '/' ) {
		$url .= "/";
	}
	
	return $url;
}



//Decodes a bEncoded string into a PHP structure
//See http://www.monduna.com/bt/faq.html#TERM_13
function bdecode($info) {
	$pos = 0;
	return bdecode_next($info,$pos);
}


//Decodes a bEncoded string, setting $pos to the the next token
function bdecode_next($info,& $pos) {

  //  print("Decoding ".substr($info,$pos,1)." at $pos <br/>\n");
    $origpos = $pos;

    switch (substr($info,$pos,1)) {

      case 'i':

	$pos = strpos($info,'e',$pos)+1;

	return substr($info,$origpos+1,$pos-$origpos-2);

      case 'd':

        $ret = array();

	$pos++;

        while (substr($info,$pos,1) != 'e') {

	  $key = bdecode_next($info,$pos);

	  $istart = $pos;

	  $val = bdecode_next($info,$pos);

	  $ilen = $pos - $istart;

	  $ret[$key] = $val;

	  if ($key == "info") {

	    $ret["sha1"] = sha1(substr($info, $istart, $ilen));

	  }



	}

	$pos++;

	return $ret;

      case 'l':

        $ret = array();

	$pos++;

        while (substr($info,$pos,1) != 'e') {

	  $ret[] = bdecode_next($info,$pos);

	}

	$pos++;

	return $ret;

      default:

	if (preg_match('/^(\d+):.*$/s',substr($info,$pos))) {

	  $len = preg_replace('/^(\d+):.*$/s','\\1',substr($info,$pos));

	  $pos += $len+1+strlen($len);

	  return substr($info,$origpos+strlen($len)+1,$pos-$origpos-strlen($len)-1);

	} else

	  return null; //There has been an error!



    }

}



//bEncodes a PHP structure
function bencode($struct) {

  if (is_array($struct)) {

    $out = 'd';

    ksort($struct);
    foreach($struct as $key => $value) {
      $out .= bencode($key);
      $out .= bencode($value);
    }

    $out .= 'e';
    return $out;
  } 
  elseif (preg_match('/^(\+|\-)?\d+$/',$struct)) {
    return 'i'.$struct.'e';
  } 
  else {
    return strlen($struct).':'.$struct;
  }
}



//
// Returns the name of the current user
//
function get_username() {

  if (isset($_SESSION['user'])) {
    return $_SESSION['user']['Username'];
  }
  else {
    return "";
  }
}

//
//This should return the current user's password or a hash of their password
//
function get_passhash() {
  if (isset($_SESSION['user']))
    return $_SESSION['user']['Hash'];
  else
    return "";
}

//
//Returns hash of password
//
function hashpass($username,$password) {
  return(sha1($username.$password."downhillb==31337"));
}



//
// logs user in
// Returns true and sets $_SESSION['user'] on success, false on error
//
function login($username,$password,&$error) {
	global $store;

	if ( ! $username || ! $password ) {
		return false;
	}

	$users = $store->getAllUsers();

	if (!isset($users[$username])) {
		$error = "User &quot;" . $username . "&quot; does not exist.";
		return false;
	} 
	elseif ($users[$username]['Hash'] != hashpass($username, $password) ) {
		$error = "Incorrect password.";
		return false;
	} 

	$users[$username]['Username'] = $username;
	$_SESSION['user'] = $users[$username];

	// this will help us track if the user can use cookies
//	$_SESSION['cookietest'] = true;

	return true;
}

//
// logs user out
//
function logout() {
	
	$_SESSION['user'] = '';
	unset($_SESSION['user']);

	return true;
}

/*
	do an HTTP AUTH login
*/
function do_http_auth() {

	if ( ! isset($_SESSION['user']) ) {
		
		// check if the current loading of the page is the first loading
		// after a logout:
		if ( isset($_SESSION['logout']) ) {
		   unset($_SESSION['logout']);
		}
		
		if (! isset($_SESSION['realm'])) {
		   $_SESSION['realm'] = site_title();
		}
		
		// check if a user has already logged in before:
		if (isset($_SESSION['user'])) {
		   unset($_SESSION['login']);
		   return true;
		}
		

		//
		// check if a user just entered a username and password:
		//
	   if ( isset($_SERVER['PHP_AUTH_USER']) && $_SERVER['PHP_AUTH_USER'] &&
			login($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'], $errstr) ) {
		   unset($_SESSION['login']);
		   return true;
		}    
		
		// let the browser ask for a username and a password:
		$_SESSION['login'] = true;
		header("WWW-Authenticate: Basic realm=\"{$_SESSION['realm']}\"");
		header("HTTP/1.0 401 Unauthorized");
		echo "You need to log in before you can access this page.";
//		return false;
		exit;
	}
	else {
		return true;
	}
}

/*
 * make sure that the user's session information hasn't been somehow spoofed,
 * that they haven't been deleted, the password changed, etc, etc.
 */
function valid_user() {

	global $store;
	$users = $store->getAllUsers();

	if (!isset($users[$_SESSION['user']['Username']])) {
		return false;
	}
	
	return true;
}

/*
 * return true if the user has admin privileges, false otherwise
 */
function is_admin() {
  return (isset($_SESSION['user']) && $_SESSION['user'] && 
  		isset($_SESSION['user']['IsAdmin']) && $_SESSION['user']['IsAdmin'] && 
		valid_user() );
}

/*
 * return true if the user has upload privileges, false otherwise
 */
function can_upload() {
  global $settings;
  return (is_admin() || 
  	(
		$settings['HasOpenChannels'] && 
		(
			(isset($_SESSION['user']) && !$_SESSION['user']['IsPending']) || !$settings['UploadRegRequired'])
		)
	);
}


/*
 * return the title of the site, if it has been specified
 */
function site_title() {
  global $settings;
  return isset($settings['Title']) ? $settings['Title'] : '';
}

/*
 * return the description of the site, if it has been specified
 */
function site_description() {
  global $settings;
  return isset($settings['Description']) ? $settings['Description'] : '';
}


function setup_data_directories() {

  global $store;
  global $seeder;

  $store = new MYSqlStore();

  if (!$store->setup()) {

    $store = new FlatFileStore();

    if (!$store->setup()) {
		bm_header();
		$store->setupHelpMessage();
		bm_footer();
		exit;
    }

  }



  $seeder = new ServerSideSeeder();
  $seeder->setup();

  return true;
}


/*
	recursive chmod which we can run when the include.php file is loaded,
	so that we can make sure that all our files have the right permissions.
	when BM is out of beta, we'll toss this.
*/
function recursive_chmod($path, $filemode) {
   if (!is_dir($path))
       return chmod($path, $filemode);

   $dh = opendir($path);
   while ($file = readdir($dh)) {
       if($file != '.' && $file != '..') {
           $fullpath = $path.'/'.$file;
           if(!is_dir($fullpath)) {
             if (!chmod($fullpath, $filemode))
                 return FALSE;
           } else {
             if (!recursive_chmod($fullpath, $filemode))
                 return FALSE;
           }
       }
   }
 
   closedir($dh);
  
   if(chmod($path, $filemode))
     return TRUE;
   else
     return FALSE;
}

/*
	check to see if all our required directories exist
*/
function check_folders() {

	global $perm_level;
	global $store;
	
	if ( ! $store ) {
		return;
	}  

	$error = array();
  	$good_perms = true;
  
	// check all of our directories to make sure we can read/write with them
	if ( $store->type = "flat file" ) {

	  $old_error_level = error_reporting(0);

		if ( ! file_exists('text') && ! mkdir('text', $perm_level ) ) {
			$error[] = "text";
		  	$good_perms = false;
		}
		else {
			recursive_chmod("text", $perm_level);
		}

		if (!file_exists('publish') && !mkdir('publish', $perm_level) ) {
			$error[] = "publish";
		  	$good_perms = false;
		}
		else {
			recursive_chmod("publish", $perm_level);
		}

		if (!file_exists( 'thumbnails') && ! mkdir( 'thumbnails', $perm_level) ) {
			$error[] = "thumbnails";
		  	$good_perms = false;
		}
		else {
			recursive_chmod("thumbnails", $perm_level);
		}

		if (!file_exists('data') && !mkdir('data', $perm_level)) {
			$error[] = "data";
		  	$good_perms = false;
		}
		else {
			recursive_chmod("data", $perm_level);
		}

		if (!file_exists('torrents') && !mkdir('torrents', $perm_level)) {
			$error[] = "torrents";
		  	$good_perms = false;
		}
		else {
			recursive_chmod("torrents", $perm_level);
		}
		
		error_reporting($old_error_level);
	}
	else if ( $store->type = "MySQL" ) {
	
	}
	
	if ( $good_perms == false ) {
		return $error;
	}

}

function check_permissions() {

	global $perm_level;
	global $store;
	
	if ( ! $store ) {
		return;
	}  

	$error = array();
  	$good_perms = true;
  
	// check all of our directories to make sure we can read/write with them
	if ( $store->type = "flat file" ) {

	  $old_error_level = error_reporting(0);

		if ( ! is_writable('text') || ! is_readable('text') ) {
			$error[] = "text";
		  	$good_perms = false;
		}
		if ( ! is_writable('publish') || ! is_readable('publish') ) {
			$error[] = "publish";
		  	$good_perms = false;
		}
		if ( ! is_writable('thumbnails') || ! is_readable('thumbnails') ) {
			$error[] = "thumbnails";
		  	$good_perms = false;
		}
		if ( ! is_writable('data') || ! is_readable('data') ) {
			$error[] = "data";
		  	$good_perms = false;
		}
		if ( ! is_writable('torrents') || ! is_readable('torrents') ) {
			$error[] = "torrents";
		  	$good_perms = false;
		}
		
		error_reporting($old_error_level);

	}
	else if ( $store->type = "MySQL" ) {
	
	}
	
	if ( $good_perms == false ) {
		return $error;
	}


}


/* cjm  - 06/03/2005 - commenting out because it's not called anywhere
function draw_first_time_link() {

  if (!file_exists('data/users')) {

    echo '<p>It looks like you\'re using Blog Torrent for the first time. You should <a href="newuser.php">create a new user account</a> before continuing.</p>';

  }

}

function draw_welcome() {

}
*/


function draw_detect_scripts() {

?>

<script  language="JavaScript" type="text/javascript" ><!--

function hasBlogTorrent() {
<?php

  //Detects IE with the client installed

if (substr_count($_SERVER['HTTP_ACCEPT'],'application/x-blogtorrent')>0) {

  echo "return true;\n";

} else {

?>

  //Detects Mozilla browser with the client installed

  for (count=0;count<window.navigator.plugins.length;count++) {

    if (window.navigator.plugins[count].name.substring(0,12) == "Blog Torrent") {

      return true;

    }

  }



  //No client is installed

  return false;

<?php } ?>

}



function hasWindows() {

    return (navigator.userAgent.indexOf('Windows') > 0);

}

function hasMac() {

    return (navigator.userAgent.indexOf('Mac') > 0);

}

--></script>

<?php

}



function draw_upload_link() {

  $user = get_username();

  $hash = get_authhash($user,get_passhash());

?>

<script  language="JavaScript" type="text/javascript" ><!--

parent.hash = "<?php echo $hash; ?>";


//
// check to see if the user has the helper installed -- if not we will send them
// a trigger link which will send along the helper with the trigger hash as well
//
if ( ! hasBlogTorrent() ) {

	if(confirm("You don't appear to have the Broadcast Machine Helper installed.  Click 'OK' to install the Broadcast Machine Helper now or 'Cancel' if you already have the Broadcast Machine Helper installed")) {

		// logic here to figure out what kind of machine the user is on
		// fixes bug # 1216625

		// if this user is on a mac, send the mac helper
		if ( hasMac() ) {
			self.location.replace('trigger.php?type=mac&hash=<?php echo $hash; ?>');
		}
		// otherwise, send the PC helper - we should have logic here for non PC/Mac people
		else if ( hasWindows() ) {
			self.location.replace('trigger.php?type=exe&hash=<?php echo $hash; ?>');
		}
		else {
			alert("It looks like you aren't using a Mac or a Windows computer - the BM Helper doesn't work on other platforms");
		}

	} 
	else {
		self.location.replace('trigger.php?hash=<?php echo $hash; ?>');
	}

}





function sendUpload() {

  if (hasBlogTorrent()) {

    document.location.replace("trigger.php?hash=<?php echo $hash; ?>");

  }

}

--></script>



<?php

}



function draw_download_link($filename) {

}



function packint32($int) {
  return pack("V",$int);
}

// Sends an installer with file $tackon on the end
// if data is not null, sends data, only uses $tackon for the name
function send_installer( $tackon, $data = null ) {
	global $store;

	$magicNumber = 560097380;
	$file = "nsisinstaller.exe";
	
	header("Content-Type: application/force-download");
	header('Content-Disposition: attachment; filename="' . basename($tackon).".exe\"");
	
//	$buffer = file_get_contents($file);

	if (is_null($data)) {
		$torrent = $store->getRawTorrent($tackon);
	}
	else {
		$torrent = $data;
	}
	
	$len = strlen($torrent);
	$crc = crc32($torrent);
	$torrent .= packint32($crc).packint32($len).packint32($magicNumber);
	
	// output $file to the browser
	readfile($file);
	
	// echo the torrent as well
	echo $torrent;
}



//Sends an installer with file $tackon on the end

// if data is not null, sends data, only uses $tackon for the name"

function send_mac_uploader($tackon,$data=null) {

  global $store;

  header("Content-type: application/octet-stream");

  header("Content-disposition: attachment; filename=BlogTorrent.zip");



  if (file_exists('macclient.obj'))

     $zip = unserialize( file_get_contents('macclient.obj'));

   else

     $zip = unserialize( file_get_contents('data/macclient.obj'));



 if (is_null($data))

   $torrent = $store->getRawTorrent($tackon);

 else

   $torrent = $data;



  $zip->add_file($torrent,'BM Helper.app/Contents/EasyUpload.blogtorrent');



  echo $zip->file();



}





//Sends an installer with file $tackon on the end

// if data is not null, sends data, only uses $tackon for the name"

function send_mac_installer($tackon,$data=null) {

  global $store;

  header("Content-type: application/octet-stream");

  header("Content-disposition: attachment; filename=BlogTorrent.zip");



  if (file_exists('macclient.obj'))

     $zip = unserialize( file_get_contents('macclient.obj'));

   else

     $zip = unserialize( file_get_contents('data/macclient.obj'));



 if (is_null($data))

   $torrent = $store->getRawTorrent($tackon);

 else

   $torrent = $data;



  $zip->add_file($torrent,'BM Helper.app/Contents/EasyDownload.torrent');



  echo $zip->file();



}



//FIXME all of the functions below this point are depricated

//      Use $store instead



function getstat($file) {
  global $store;
  $store->getStat($file);
}



//The following functions generate and verify authentication hashes
//given to the client.  Using authentication hashes lets us keep all
//of the authentication on the server side. The client only has to
//keep the hash, then send it back. It never has to deal with
//passwords. The "hash" file is a bEncoded associative array.  The
//indexes to the array are authentication hashes and usernames hashed
//together. The values are timestamps.



//We hash the authentication hash with the username, so that if

//someone downloads the hash file somehow, they can't just copy the

//hashes and upload files without authentication.



function is_valid_authhash($username,$hash) {

  global $store;

  return $store->isValidAuthHash($username,$hash);

}



function get_authhash($username, $passhash) {
  global $store;
  return $store->getAuthHash($username,$passhash);
}



function drop_authhash($username,$hash,$torrent) {

  global $store;

  return $store->dropAuthHash($username,$hash,$torrent);

}



//Private function used by add_hash and drop_hash

function clear_old_authhashes(& $hashes) {

  global $store;

  $store->clearOldAuthHashes($hashes);

}



//Sends the location of this feed to BlogTorrent.com periodically

function phone_home() {

  global $store;

  return $store->phoneHome();

}



function add_torrent_to_tracker($filename) {

  global $store;

  return $store->addTorrentToTracker($filename);

}





function get_all_users () {
  global $store;
  return $store->getAllUsers();
}



function delete_torrent($torrent) {

  global $store;

  $store->deleteTorrent($torrent);

}



function delete_user($username) {

  global $store;

  return $store->deleteUser($username);

}



//New users are stored as a list of hashes

function add_new_user($username,$password,$email, $isAdmin, $isFront, &$error) {

  global $store;

  return $store->addNewUser($username,$password,$email, $isAdmin, $isFront, $error);
//  return $store->addNewUser($username,$password,$email, $isAdmin, $isFront, &$error);

}



function auth_new_user($hashlink,$username) {

  global $store;

  return $store->authNewUser($hashlink,$username);

}



function change_password($password) {

  global $store;

  return $store->changePassword($password);

}



//Adds/Updates a user in the user database

function update_user($username,$hash,$email,$canUpload=false,$canAdmin=false) {

  global $store;

  return $store->updateUser($username,$hash,$email,$canUpload,$canAdmin);

}



//Loads settings from config file

//Returns true on success, false on failure

function load_settings() {

  global $store;

  return $store->loadSettings();

}



//Saves settings to config file

//Returns true on success, false on failure

function save_settings($newsettings) {

  global $store;

  return $store->saveSettings($newsettings);

}




function front_header($pagename = "", $channelID = "", $stylesheet = "default.css", $feed = "" ) {

	global $settings;

	print("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">
<html>
<head>
<title>");

	if ($pagename) {
		print($pagename);
	}

	print("</title>
<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />
<link rel=\"stylesheet\" type=\"text/css\" href=\"" . $stylesheet . "\"/>");

	if ( $feed != "" ) {
		print "<link rel=\"alternate\" type=\"application/rss+xml\" title=\"RSS 2.0\" href=\"$feed\" />";
	}

	print("</head>
<body>
<div id=\"wrap\">
<div id=\"inner_wrap\">
<div id=\"login_links\">");

	if (!(isset($_SESSION['user']) && is_array($_SESSION['user']))) {

		print("<a href=\"login.php?f=1\">Login</a> | 
				<a href=\"newuser.php?f=1\">Register</a>");

	} 
	else {

		global $can_use_cookies;
		if ( $can_use_cookies ) {
			print("<a href=\"login.php?f=1&logout=1\">Logout</a> ");
		}
		
		if ( is_admin() ) {
			print("| <a href=\"admin.php\">Admin</a>");		
		}

	}

	if (can_upload()) {

		print(" | <a href=\"publish.php\">Post a File</a>");

	}

	print("</div><div id=\"library_header_wrap\"><div id=\"library_title\">" . $pagename . " (Beta)</div>");

	if ( $channelID ) {
		$rsslink = '<a href="rss.php?i=' . $channelID . '"><img src="images/rss_button.gif" alt="rss feed" border="0" /></a>';
//		print("<div id=\"rss_feed\"><a href=\"rss.php?i=" . $channelID . "\">RSS Feed</a></div>");
		print("<div id=\"rss_feed\">$rsslink</div>");
	}

	print("</div>");

}

function front_footer() {

	print("<div id=\"footer\">Powered by <a href=\"http://www.participatoryculture.org/bm/\">Broadcast Machine</a></div></div></div><br/></body></html>");

}

function bm_header($pagename = "") {

	global $settings;

	print <<<EOF
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
<title>Broadcast Machine
EOF;

	if ($pagename) {
		print(" - " . $pagename);
	}

	print <<<EOF
</title>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<link rel="stylesheet" type="text/css" href="pub_css.css"/>
</head>
<body>
<div id="head">
<h1><div id="preview">Preview Version</div></h1>
</div>
<div id="logged_in">

EOF;

	if (!(isset($_SESSION['user']))) {

		print("<a href=\"login.php\">Sign In</a>");
		if ( allowAddNewUser() ) {
			print(" | <a href=\"newuser.php\">Sign Up</a>");
		}

	} else {

		print("<span style=\"color: #444\"><a href=\"index.php\">View Front Page &gt;&gt;</a> ");
		if ( isset($_SESSION['user']['Name']) ) { 
			print( "| <strong><a href=\"user_edit.php?i=" . $_SESSION['user']['Name']  . "\">" . $_SESSION['user']['Name'] . "</a></strong>");
			global $can_use_cookies;
			
			// if we're using DTV or a cookieless browser, logging out doesn't work
			if ( $can_use_cookies == true ) {
				print " - <a href=\"login.php?logout=1\">logout</a>";
			}
		}
	}

	print("</div><div id=\"adminmenu\"><div id=\"inner_nav\"><a href=\"admin.php\">Dashboard</a>");

	if (can_upload()) {

		print("<a href=\"publish.php\">Publish</a><a href=\"edit_videos.php\">Files</a>");

	} else {

		print("<a href=\"#\" onClick=\"return false;\" style=\"color:#999999\">Files</a><a href=\"#\" onClick=\"return false;\" style=\"color:#999999\">Library</a>" );

	}

	if (is_admin()) {

		print("<a href=\"channels.php\">Channels</a><a href=\"settings.php\">Settings</a><a href=\"users.php\">Users</a>");

	} else {

		print("<a href='#' onClick=\"return false;\" style=\"color:#999999\">Channels</a><a href='#' onClick=\"return false;\" style=\"color:#999999\">Settings</a><a href=\"#\" onClick=\"return false;\" style=\"color:#999999\">Users</a>");

	}

	print("<a href=\"http://www.participatoryculture.org/bm/help/\" target=\"_blank\"  style=\"color: #B55;\">Help</a></div></div>");

}



function bm_footer() {

	print("<div class=\"spacer\">&nbsp;</div></body></html>");

}



function display_video($filehash, $file) {

	global $channel;

	if ($file["Publishdate"] <= time()) {

print("<li>");

       	if ($channel['Options']['Thumbnail']) {
        	
			print("<div class=\"thumbnail\"><a href=\"detail.php?c=" . $channel["ID"] . "&i=" . $filehash  . "\"><img src=\"");
	
			if ($file['Image'] == '' || !$channel['Options']['Thumbnail']) {
				print("t.gif");
			} else {
				print($file['Image']);
			}
			print("\" width=\"180\" border=\"0\"/></a></div>"); 
		}

		if ($channel['Options']['Title']) {

			print("<div class=\"video_title\"><a href=\"detail.php?c=" . $channel["ID"] . "&i=" . $filehash . "\">" . $file["Title"] . "</a></div>");

		}

		print("<div class=\"video_description\">");

		if ($channel['Options']['Desc'] && $file["Desc"] != "") {

			print(substr($file["Desc"],0,52) . "... ");

		}

		print("<a href=\"detail.php?c=" . $channel["ID"] . "&i=" . $filehash  . "\">more...</a></div>");
		print ("<div class=\"dl_links\">Get it: <a href=\"download.php?c=" . $channel["ID"] . "&i=" . $filehash  . "\">download</a></div></li>");

	}

}

function allowAddNewUser() {
	global $settings;
	return $settings['AllowRegistration'];
}

function requireUserAccess($do_login = false) {

	$got_user = true;

	if ( ! isset($_SESSION['user']) || (
		isset($_SESSION['user']['IsPending']) && $_SESSION['user']['IsPending'] == true ) ) {

		$got_user = false;
	}
	else if ( !valid_user() ) {
		$got_user = false;
	}

	if ( ! $got_user ) {
		if ( $do_login == true ) {
			header('Location: ' . get_base_url() . 'login.php' );
		}
		else {
			header('Location: ' . get_base_url() . 'index.php' );
		}
		exit;
	}

}
function requireUploadAccess() {

	if (!can_upload()) {
		header('Location: ' . get_base_url() . 'admin.php');
		exit;
	}

}

function requireAdminAccess() {

	global $requireAdmin;

	if ( !is_admin() && $requireAdmin ) {
		return false;
	}
	
	return true;
}

function comp($a, $b) {
	return ($b[1] - $a[1]);
}

// cjm - make sure that output was stripped of html (1201432)
// and also encoded in utf-8 (1201431)  - 05/23/2005
function encode($s) {
	return utf8_encode(htmlentities($s, ENT_NOQUOTES, "UTF-8"));
}

function get_filesize($tmpurl) {

	//
	// if this is a local file, then let's get the size info from the filesystem
	//
	if ( is_local_file($tmpurl) && is_local_torrent($tmpurl) ) {
		$filename = local_filename($tmpurl);
		$stats = bdecode(file_get_contents("torrents/" .$filename));
		
		if ( isset($stats["info"]["length"]) ) {
			$length = $stats["info"]["length"];
		}
		else {
			$length = 0;
		}
	}
	else {
		$length = get_content_length($tmpurl, $errstr);
	}
	
	return $length;

}

function makeChannelRss($channelID) {
	
	global $store;
	global $perm_level;
	
	if ( ! $channelID || $channelID == "" ) {
		return;
	}

	if (!file_exists('publish')) {
		mkdir("publish",$perm_level);
	}
	if (!file_exists('publish/' . $channelID)) {
		mkdir("publish/" . $channelID, $perm_level);
	}

/*	$use_cache = true;
	
	if (!file_exists("publish/" . $channelID . ".rss")) {
		$use_cache = false;
	}
	*/
	
//	$last_rss_update = 0;
	$this_dir = dir("publish/" . $channelID);

	while(($file = $this_dir->read()) !== false) {
	
		if (is_numeric($file)) {
		
//			$last_rss_update = $file;
		
			if (time() > $file) {
//				$use_cache = false;
				unlink("publish/" . $channelID . "/" . $file);
			}
		}
	}
	
	// also force an rss rebuild if we've updated our rss-generation code
//	if ( filemtime("include.php") > $last_rss_update ) {
//		$use_cache = false;
//	}

	$channels = $store->getAllChannels();
	$channel = $channels[$channelID];
	$files = $store->getAllFiles();

	$base_url=get_base_url();


	$sOut = '<?xml version="1.0" encoding="utf-8"?>';

// cjm removed on 05-19-2005 - bug #1205109
// <!DOCTYPE rss [<!ENTITY % HTMLlat1 PUBLIC "-//W3C//ENTITIES Latin 1 for XHTML//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml-lat1.ent">]>

// cjm changed from this namespace to mrss.  i think this is right according to
// http://search.yahoo.com/mrss 05-19-2005 - bug #1201605
//	xmlns:media="http://docs.yahoo.com/mediaModule" 


	$sOut .= <<<EOF
<rss version="2.0" 
	xmlns:media="http://search.yahoo.com/mrss" 
	xmlns:creativeCommons="http://backend.userland.com/creativeCommonsRssModule">

EOF;

	$sOut .= "<channel><title>" . encode($channel['Name']) . "</title>";
	$sOut .= "<description>" . encode($channel['Description']) . "</description>";
	$sOut .= "<link>" . htmlspecialchars($channel['LibraryURL']) . "</link>";
	$sOut .= "<pubDate>" . date('r', time()) . "</pubDate>";
	$sOut .= "<generator>Broadcast Machine Preview" . version_number() . "</generator>";
	$sOut .= "<atom:link 
   rel=\"self\" 
   type=\"application/rss+xml\" 
   title=\"" . encode($channel['Name']) . "\" 
   href=\"" . get_base_url() . "rss.php?i=$channelID\" 
   xmlns:atom=\"http://purl.org/atom/ns#\" />";

	$channel_files = $channel["Files"];
	usort($channel_files, "comp");

	$files = $store->getAllFiles();

	foreach ($channel_files as $file) {
	
		if (array_key_exists($file[0], $files)) {

			$data = $files[$file[0]];

			if ($data["Publishdate"] <= time()) {
				$sOut .= '<item><title>' . encode($data['Title']) . '</title>
				<link>' . $base_url . 'detail.php?c=' . $channel["ID"] . '&amp;i=' . $file[0] . '</link>
				<guid isPermaLink="false">' . $file[0] . '</guid>';

				// dont send a description flag if we don't have the data
				if ( $data["Desc"] ) {
					$sOut .= '<description>' . encode($data['Desc']) . '</description>';
				}

				// cjm - get the length - is this right?
				$tmpurl = $files[$file[0]]["URL"];
				$length = get_filesize($tmpurl);
				
				// ensure we have a positive length, which we need to produce valid rss
				if ( $length <= 0 ) {
					$length = 1;
				}


				$sOut .= '<enclosure 
						url="' . $base_url . 'download.php?c=' . $channelID . '&amp;i=' . $file[0] . '" ';

				// sometimes the mime-type comes with some extra data (like the charset)
				// make sure we remove that here
				if ( $data["Mimetype"] ) { 
					$junk = split(";", $data["Mimetype"]);
					$sOut .= 'type="' . $junk[0] . '" ';
				}
				// ensure we have a mime-type here.  this might not be ideal,
				// but we need this to produce valid rss
				else {
					$sOut .= 'type="application/octet-stream" ';
				}
				
				$sOut .= ' length="' . $length . '" ';
				
				$sOut .= '/>
						<pubDate>' . date("D, d M Y H:i:s O", $data["Publishdate"]) . '</pubDate>';

	
				// dont send if we dont have a thumbnail
				if ( $data["Image"] ) {
					$sOut .= '<media:thumbnail url="' . $data["Image"] . '" />';
				}
	
				foreach ($data["People"] as $people) {
					if ( $people[0] && $people[1] ) {
						$sOut .= "<media:people 
							role=\"" . encode($people[1]) . "\">" . 
							encode($people[0]) . "</media:people>";
					}
				}
	
				$sOut .= '</item>';
	
			}
	
		}
	
	}
		
		// if we have a thumbnail, go ahead and display it
	if ( isset($channel["Icon"]) && $channel["Icon"] != "" ) {
		$sOut .= "
<image>
<title>" . encode($channel['Name']) . "</title>
<url>" . $channel["Icon"] . "</url>
<link>" . htmlspecialchars($channel['LibraryURL']) . "</link>
</image>";
	}

	$sOut .= '</channel>';
	$sOut .= '</rss>';

	$handle = fopen("publish/" . $channelID . ".rss", "a+b");
	flock($handle,LOCK_EX);
	fseek($handle,0);
	ftruncate($handle,0);
	fwrite($handle,$sOut);
	fclose($handle);
}

/*
 code to handle returning a 304 if our RSS hasn't changed
 from: http://simon.incutio.com/archive/2003/04/23/conditionalGet
*/
function doConditionalGet($timestamp) {
    // A PHP implementation of conditional get, see 
    //   http://fishbowl.pastiche.org/archives/001132.html
    $last_modified = substr(date('r', $timestamp), 0, -5).'GMT';
    $etag = '"'.md5($last_modified).'"';
    // Send the headers
    header("Last-Modified: $last_modified");
    header("ETag: $etag");
    // See if the client has provided the required headers
    $if_modified_since = isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) ?
        stripslashes($_SERVER['HTTP_IF_MODIFIED_SINCE']) :
        false;
    $if_none_match = isset($_SERVER['HTTP_IF_NONE_MATCH']) ?
        stripslashes($_SERVER['HTTP_IF_NONE_MATCH']) : 
        false;
    if (!$if_modified_since && !$if_none_match) {
        return;
    }
    // At least one of the headers is there - check them
    if ($if_none_match && $if_none_match != $etag) {
        return; // etag is there but doesn't match
    }
    if ($if_modified_since && $if_modified_since != $last_modified) {
        return; // if-modified-since is there but doesn't match
    }
    // Nothing has changed since their last request - serve a 304 and exit
    header('HTTP/1.0 304 Not Modified');
    exit;
}

function displayChannelRSS($channelID) {

	$rss_file = "publish/" . $channelID . ".rss";

	if (!file_exists($rss_file)) {
		makeChannelRss($channelID);
	}

	header('Content-Type: application/rss+xml; charset=utf-8');

	// see if the file has been modified or not.  if not, this function
	// will return a 304 code and exit processing
	$filedate = gmdate("D, d M Y H:i:s", filemtime($rss_file)) . " GMT";
	doConditionalGet($filedate);

	// otherwise, return the file
	print( file_get_contents($rss_file));
}

// returns true if $str begins with $sub
function beginsWith( $str, $sub ) {
	$str = strtolower($str);
	$sub = strtolower($sub);
   return ( substr( $str, 0, strlen( $sub ) ) == $sub );
}

// return true if $str ends with $sub
function endsWith( $str, $sub ) {
	$str = strtolower($str);
	$sub = strtolower($sub);
   return ( substr( $str, strlen( $str ) - strlen( $sub ) ) == $sub );
}

if(!function_exists('get_headers')) {
  
   /**
   * @return array
   * @param string $url
   * @param int $format
   * @desc Fetches all the headers
   * @author cpurruc fh-landshut de
   * @modified by dotpointer
   * @modified by aeontech
   * grabbed from http://us2.php.net/get_headers
   */
   function get_headers( $url, $format = 0 ) {
   
   		// make sure we don't die on a bad URL here
   		$old_error_level = error_reporting(0);
   
       $url_info = parse_url($url);
	   
	   if ( $url_info["scheme"] != "http" ) {
			return false;
	   }
	   
       $port = isset($url_info['port']) ? $url_info['port'] : 80;
       $fp = fsockopen($url_info['host'], $port, $errno, $errstr, 15);
      
       if($fp) {
           $head = "HEAD ".@$url_info['path']."?".@$url_info['query']." HTTP/1.0\r\nHost: ".@$url_info['host']."\r\n\r\n";     
           fputs($fp, $head);     
           while(!feof($fp))
           {
               if($header=trim(fgets($fp, 1024)))
               {
                   if($format == 1)
                   {
                       $key = array_shift(explode(':',$header));
                       // the first element is the http header type, such as HTTP 200 OK,
                       // it doesn't have a separate name, so we have to check for it.
                       if($key == $header)
                       {
                           $headers[] = $header;
                       }
                       else
                       {
					   		// lowercase the key so we don't have to deal with any bizzare server responses
                           $headers[strtolower($key)]=substr($header,strlen($key)+2);
                       }
                       unset($key);
                   }
                   else
                   {
                       $headers[] = $header;
                   }
               }
           }

			// if these headers included a redirect, then let's recurse down
			// to the file the user would be sent to, and get its headers
		   if ( isset($headers["location"]) ) {
				$redirect = $headers["location"];

				// make sure we have an absolute URL
				if ( beginsWith($redirect, "http") == false ) {
					$redirect = $url_info["scheme"] . "://" . $url_info["host"] . $redirect;
				}

				return get_headers($redirect, 1);
		   }

			error_reporting($old_error_level);
           return $headers;
       }
       else
       {
			error_reporting($old_error_level);
           return false;
       }
   }
}


function get_content_length( $file_url, &$errstr ) {

	$headers = get_headers($file_url, 1);
	
	if ( ! $headers ) {
		$errstr = "404";
		return 0;
	}
	
	if ( isset($headers["content-length"]) ) {
		return $headers["content-length"];
	}
	else {
		return 0;
	}
}


function get_content_type( $file_url, &$errstr ) {

	if ( is_local_file($file_url) ) {
		$fname = local_filename($file_url);
		return mime_content_type("torrents/" . $fname);
	}
	else {

		$headers = get_headers($file_url, 1);
		
		if ( ! $headers ) {
			$errstr = "404";
			return "";
		}
		
		return $headers["content-type"];
	}
}

//
// from http://us4.php.net/manual/en/function.rawurlencode.php comments
//
function linkencode($p_url){
   $uparts = @parse_url($p_url);

   $scheme = array_key_exists('scheme',$uparts) ? $uparts['scheme'] : "";
   $pass = array_key_exists('pass',$uparts) ? $uparts['pass']  : "";
   $user = array_key_exists('user',$uparts) ? $uparts['user']  : "";
   $port = array_key_exists('port',$uparts) ? $uparts['port']  : "";
   $host = array_key_exists('host',$uparts) ? $uparts['host']  : "";
   $path = array_key_exists('path',$uparts) ? $uparts['path']  : "";
   $query = array_key_exists('query',$uparts) ? $uparts['query']  : "";
   $fragment = array_key_exists('fragment',$uparts) ? $uparts['fragment']  : "";

   if(!empty($scheme))
     $scheme .= '://';

   if(!empty($pass) && !empty($user)) {
     $user = rawurlencode($user).':';
     $pass = rawurlencode($pass).'@';
   } elseif(!empty($user))
     $user .= '@';

   if(!empty($port) && !empty($host))
       $host = ''.$host.':';
   elseif(!empty($host))
       $host=$host;

   if(!empty($path)){
     $arr = preg_split("/([\/;=])/", $path, -1, PREG_SPLIT_DELIM_CAPTURE); // needs php > 4.0.5.
     $path = "";
     foreach($arr as $var){
       switch($var){
       case "/":
       case ";":
       case "=":
         $path .= $var;
         break;
       default:
         $path .= rawurlencode($var);
       }
     }
     // legacy patch for servers that need a literal /~username
     $path = str_replace("/%7E","/~",$path);
   }

   if(!empty($query)){
     $arr = preg_split("/([&=])/", $query, -1, PREG_SPLIT_DELIM_CAPTURE); // needs php > 4.0.5.
     $query = "?";
     foreach($arr as $var){
       if( "&" == $var || "=" == $var )
         $query .= $var;
       else
         $query .= urlencode($var);
     }   
   }

   if(!empty($fragment))
     $fragment = '#'.urlencode($fragment);

   return implode('', array($scheme, $user, $pass, $host, $port, $path, $query, $fragment));
}



function displayTorrentInfo( $url, $filehash, $return_url, $type = "basic", $restarted = false ) {

	global $store;
	global $seeder;
	global $settings;

	$files = $store->getAllFiles();
	$file = $files[$filehash];

//	$torrentfile = urldecode(substr($url,strlen(get_base_url()) + 9));
	$torrentfile = local_filename($url);
	$torrenthash = $store->getHashFromTorrent($torrentfile);
	$stats = $store->getStat($torrenthash);
	$details = $seeder->getSpawnStatus($torrenthash);

	$pause_url = "pause.php?i=" . $filehash  . "&return_url=" . urlencode($return_url);
	$start_url = "start.php?i=" . $filehash  . "&return_url=" . urlencode($return_url);
	$stop_url = "stop.php?i=" . $filehash  . "&return_url=" . urlencode($return_url);


	if ( ( isset($settings['sharing_enable']) && $settings['sharing_enable'] == 1 ) ) {
		$sharing = true;
	}
	else {
		$sharing = false;
	}
	
//	if ( ! ( isset($settings['sharing_enable']) && $settings['sharing_enable'] == 1 ) ) {
		$file["SharingEnabled"] = false;
//	}
	//else 
	if ( isset($settings['sharing_enable']) && isset($settings['sharing_auto']) && 
		$settings['sharing_enable'] == 1 && $settings['sharing_auto'] == 1 ) {
		$file["SharingEnabled"] = true;
	}

	print("<div class=\"video_stats\">");

//	if ( $type == "basic" ) {

		// we should always be showing this info
		print "
			<strong>Torrent Status</strong>
			<br />
			SEEDERS: " . $stats["complete"] . "<br />
			DOWNLOADERS: " . $stats["incomplete"] . "<br /><br />";
	
	//}
	//else 
	if ( $type != "basic" && $sharing == true ) {
		if ( !isset($file["SharingEnabled"]) || $file["SharingEnabled"] == false ) {
			print "<strong>Server Sharing STOPPED</strong><br />
				<a href=\"$start_url\">Start</a><br />";	
		}	
		else if ( isset($details["time left"]) && trim($details["time left"]) == "shutting down" ) {
			print "<strong>Server Sharing PAUSED</strong><br />
				<a href=\"$start_url\">Start</a> | <a href=\"$stop_url\">Stop</a><br /><br />";
		}
		else if ( isset($details["Running"]) && $details["Running"] == 1 ) {
	
			$details["percent done"] = trim($details["percent done"]);
	
			if ( $details["percent done"] == "100.0" ) {
				$status = "seeding file";
			}
			else {
				$status = "downloading file to server: " . $details["percent done"] . "%";
			}
		
			$teststr = $details["share rating"];
			$teststr = str_replace(" (", "|", $teststr);
			$teststr = str_replace(" MB up / ", "|", $teststr);
			$teststr = str_replace(" MB down)", "", $teststr);
		
			$bandwidth = explode("|", $teststr);
			if ( isset($bandwidth[1]) && isset($bandwidth[2]) ) {
				$tally = $bandwidth[1] + $bandwidth[2];
			}
			else {
				$tally = "0.0";
			}
	
	/*			<strong>Torrent Status</strong>
				<br />
				SEEDERS: " . $stats["complete"] . "<br />
				DOWNLOADERS: " . $stats["incomplete"] . "<br /><br />*/
			print "
				<strong>Server Sharing ON</strong><br />
				Status: " . $status . "<br />
				<a href=\"$pause_url\">Pause</a> | <a href=\"$stop_url\">Stop</a><br /><br />
				Bandwidth Used: " . $tally . " MB";
		}
		else {
			print "<strong>Server Sharing STOPPED</strong><br />";
			
			if ( $restarted ) {
				print "(Restarting)<br />";
			}
			else {
				print "<a href=\"$start_url\">Start</a><br />";	
			}
		}

	} // if ( not basic display )

	print "</div>";

}

function is_local_file($url) {

	// figure out what server we are one - since URLs sometimes have www
	// and sometimes don't, make sure we don't include that in the search
	$my_server = str_replace("www.", "", $_SERVER["HTTP_HOST"]);

	if ( ( beginsWith($url, "http://" ) || beginsWith($url, "https://" ) ) &&
		stristr( $url, $my_server ) && 
		stristr( $url, "/torrents/") ) {
		return true;
	}

	return false;
}



/*
 * figure out if this is a .torrent file located within the control of BM.  
 * if so, we can share it
 */
function is_local_torrent($url) {

//	if ( beginsWith($url, get_base_url() ) && endsWith( $url, ".torrent" ) ) {
//	$filename = substr($file["URL"], strlen(get_base_url() . "torrents/"));

	if ( is_local_file($url) && endsWith( $url, ".torrent" ) ) {
		return true;
	}
	
	return false;
}

/*
 *
 * figure out the name of a local file, given it's URL
 *
 */
function local_filename($url) {

	if ( is_local_file($url) ) {
		$filename = str_replace("www.", "", $url );
		
		$base = get_base_url() . "torrents/";
		$base = str_replace("www.", "", $base );
	
		$filename = substr($filename, strlen($base));

		return $filename;
	}
	else {
		return false;
	}

}
?>